
from agentevals.trajectory.llm import (
    TRAJECTORY_ACCURACY_PROMPT,
    create_trajectory_llm_as_judge,
)
from google.ai.generativelanguage_v1beta import ToolConfig
from langsmith.evaluation import EvaluationResult, run_evaluator
from langsmith.schemas import Example, Run
from openevals.llm import create_llm_as_judge
from openevals.prompts import CORRECTNESS_PROMPT


@run_evaluator
def exact_match_evaluator(run: Run, example: Example | None = None) -> EvaluationResult:
    """
    A simple evaluator that checks for exact match between the agent's output
    and the expected output from the dataset.
    """
    if example is None or "expected_output" not in example.outputs:
        return EvaluationResult(
            key="exact_match", score=0, comment="No expected output provided. Example: " + str(example)
        )

    # The agent's response might be in a list of messages
    agent_response_raw = run.outputs.get("output", "")
    if isinstance(agent_response_raw, list):
        # Extract text from the last dictionary in the list
        agent_response_raw = agent_response_raw[-1]


    final_answer = agent_response_raw.get("content", "").strip().lower()

    expected_output = example.outputs["expected_output"].strip().lower()
    if final_answer == expected_output:
        score = 1
        comment = "Exact match."
    else:
        score = 0
        comment = f"Mismatch: Expected '{expected_output}', but got '{final_answer}'."

    return EvaluationResult(key="exact_match", score=score, comment=comment)


correctness_evaluator = create_llm_as_judge(
    prompt=CORRECTNESS_PROMPT,
    feedback_key="correctness",
    model="anthropic:claude-4-sonnet-20250514",
)


trajectory_evaluator = create_trajectory_llm_as_judge(
    prompt=TRAJECTORY_ACCURACY_PROMPT,
    model="anthropic:claude-4-sonnet-20250514",
)

@run_evaluator
def tool_node_evaluator(run: Run, example: Example | None = None) -> EvaluationResult:
    """
    A simple evaluator that checks if the agent used the required tools.
    """
    try:
        if example is None or example.outputs is None or "required_tools" not in example.outputs:
            return EvaluationResult(key="tool_node", score=0, comment="No required tools provided. Example: " + str(example))
        required_tools : ToolConfig = example.outputs["required_tools"]
        agent_response_raw : ToolConfig = run.outputs.get("tool_config", {})
        # Flatten the tool_configs to a single set of tool_ids
        required_tool_ids = [f"{app_id}___{tool_id}" for app_id, tools in required_tools.items() for tool_id in tools]
        agent_tool_ids = [f"{app_id}___{tool_id}" for app_id, tools in agent_response_raw.items() for tool_id in tools]
        if set(required_tool_ids).issubset(set(agent_tool_ids)):
            return EvaluationResult(key="tool_node", score=1, comment="Tool usage: " + str(required_tools))
        else:
            return EvaluationResult(key="tool_node", score=0, comment="Tool usage: " + str(required_tools))
    except Exception as e:
        print(f"Error evaluating tool usage: {str(e)}")
        print(run.outputs)
        print(example.outputs)
        return EvaluationResult(key="tool_node", score=0, comment=f"Error evaluating tool usage: {str(e)}")


import argparse
import asyncio
from typing import Any

from langsmith import Client, aevaluate
from langsmith.evaluation import RunEvaluator
from universal_mcp.agentr.client import AgentrClient
from universal_mcp.agentr.registry import AgentrRegistry

from evals.dataset import load_dataset
from evals.evaluators import (
    correctness_evaluator,
    exact_match_evaluator,
    trajectory_evaluator,
    tool_node_evaluator,
)
from universal_mcp.agents import get_agent
from universal_mcp.agents.base import BaseAgent
from universal_mcp.agents.utils import messages_to_list


# 1. Agent Factory
def build_agent(agent_name: str):
    """
    Factory function to get an agent instance by name.
    """
    client = AgentrClient()
    common_params = {
        "instructions": "You are a helpful assistant. Respond to the final answer in one or two words. Eg, if the answer is 4, you should respond with '4'. Do not provide with any explanation",
        "model": "anthropic/claude-4-sonnet-20250514",
        "registry": AgentrRegistry(client=client) if agent_name != "simple" else None,
    }
    agent = get_agent(agent_name)(name=agent_name, **common_params)
    return agent



# 2. Evaluator Registry
EVALUATORS: dict[str, Any] = {
    "llm_as_judge": correctness_evaluator,
    "exact_match": exact_match_evaluator,
    "trajectory": trajectory_evaluator,
    "tool_node": tool_node_evaluator,
}


def get_evaluator(evaluator_name: str) -> RunEvaluator:
    """
    Retrieves an evaluator from the registry.
    """
    evaluator = EVALUATORS.get(evaluator_name)
    if evaluator is None:
        raise ValueError(
            f"Unknown evaluator: {evaluator_name}. Available evaluators: {', '.join(EVALUATORS.keys())}"
        )
    return evaluator



async def agent_runner(agent: BaseAgent, inputs: dict) -> dict:
    """
    Runs the agent and returns a dictionary with the final output.
    """
    result = await agent.invoke(user_input=inputs["user_input"])
    messages = messages_to_list(result["messages"])
    return_result = {"output": messages}
    if "tool_config" in result:
        return_result["tool_config"] = result["tool_config"]
    return return_result

async def main(agent_name: str, dataset_path: str, evaluator_name: str):
    """
    The main function for the evaluation CLI.
    """

    # 1. Get the agent and evaluator
    agent = build_agent(agent_name)
    evaluator = get_evaluator(evaluator_name)

    # Create a callable for aevaluate
    async def target_func(inputs: dict):
        return await agent_runner(agent, inputs)

    # 2. Load the dataset from file
    dataset_examples = load_dataset(dataset_path)

    # 3. Upload dataset to LangSmith for the evaluation run
    client = Client()
    dataset_name = dataset_path.split("/")[-1].split(".")[0]
    # dataset_name = f"{agent_name}-{evaluator_name}-eval-dataset"
    try:
        # If dataset with same name and examples exists, read it.
        # Otherwise, a new one is created.
        dataset = client.create_dataset(
            dataset_name,
            description=f"Dataset for {agent_name} evaluation with {evaluator_name}.",
        )
        for example in dataset_examples:
            client.create_example(
                inputs={"user_input": example["user_input"]},
                outputs={
                    "expected_output": example.get("expected_output", ""), 
                    "required_tools": example.get("required_tools", {})
                },
                dataset_id=dataset.id,
            )
    except Exception:
        pass

    # 4. Run the evaluation
    await aevaluate(
        target_func,
        data=dataset_name,  # Pass the dataset name
        evaluators=[evaluator],
        experiment_prefix=f"{agent_name}-{evaluator_name}-eval",
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run evaluations on different agents.")
    parser.add_argument(
        "agent",
        type=str,
        help="The name of the agent to evaluate.",
    )
    parser.add_argument(
        "dataset",
        type=str,
        help="Path to the dataset file (e.g., src/evals/example_dataset.jsonl).",
    )
    parser.add_argument(
        "evaluator",
        type=str,
        choices=list(EVALUATORS.keys()),
        help="The name of the evaluator to use.",
    )
    args = parser.parse_args()

    asyncio.run(
        main(
            agent_name=args.agent,
            dataset_path=args.dataset,
            evaluator_name=args.evaluator,
        )
    )



from langgraph.checkpoint.base import BaseCheckpointSaver
from universal_mcp.tools.registry import ToolRegistry

from universal_mcp.agents.base import BaseAgent
from universal_mcp.agents.llm import load_chat_model
from universal_mcp.agents.react import ReactAgent

from .graph import build_graph
from .prompts import DEVELOPER_PROMPT


class PlannerAgent(BaseAgent):
    def __init__(
        self,
        name: str,
        instructions: str,
        model: str,
        registry: ToolRegistry,
        memory: BaseCheckpointSaver | None = None,
        executor_agent_cls: type[BaseAgent] = ReactAgent,
        **kwargs,
    ):
        super().__init__(name, instructions, model, memory, **kwargs)
        self.app_registry = registry
        self.llm = load_chat_model(model)
        self.executor_agent_cls = executor_agent_cls

    def _build_system_message(self):
        return DEVELOPER_PROMPT.format(
            name=self.name, instructions=self.instructions
        )

    async def _build_graph(self):
        return build_graph(
            self.llm,
            self.app_registry,
            self._build_system_message(),
            self.model,
            self.executor_agent_cls,
        ).compile(checkpointer=self.memory)

    @property
    def graph(self):
        return self._graph


__all__ = ["PlannerAgent"]



import asyncio

from loguru import logger

from universal_mcp.agentr.registry import AgentrRegistry
from universal_mcp.agents.bigtool2 import BigToolAgent2
from universal_mcp.agents.utils import messages_to_list


async def main():
    agent = BigToolAgent2(
        name="bigtool",
        instructions="You are a helpful assistant that can use tools to help the user.",
        model="azure/gpt-4.1",
        registry=AgentrRegistry(),
    )
    await agent.ainit()
    output = await agent.invoke(
        user_input="Send an email to manoj@agentr.dev"
    )
    logger.info(messages_to_list(output["messages"]))


if __name__ == "__main__":
    asyncio.run(main())



from langgraph.checkpoint.base import BaseCheckpointSaver
from universal_mcp.logger import logger
from universal_mcp.tools.registry import ToolRegistry

from universal_mcp.agents.base import BaseAgent
from universal_mcp.agents.llm import load_chat_model

from .graph import build_graph
from .prompts import SYSTEM_PROMPT


class BigToolAgent(BaseAgent):
    def __init__(
        self,
        name: str,
        instructions: str,
        model: str,
        registry: ToolRegistry,
        memory: BaseCheckpointSaver | None = None,
        **kwargs,
    ):
        super().__init__(name, instructions, model, memory, **kwargs)
        self.registry = registry
        self.llm = load_chat_model(self.model)

        logger.info(
            f"BigToolAgent '{self.name}' initialized with model '{self.model}'."
        )

    def _build_system_message(self):
        return SYSTEM_PROMPT.format(
            name=self.name,
            instructions=self.instructions,
        )

    async def _build_graph(self):
        """Build the bigtool agent graph using the existing create_agent function."""
        logger.info(f"Building graph for BigToolAgent '{self.name}'...")
        try:
            graph_builder = build_graph(
                tool_registry=self.registry,
                llm=self.llm,
                system_prompt=self._build_system_message(),
            )

            compiled_graph = graph_builder.compile(checkpointer=self.memory)
            logger.info("Graph built and compiled successfully.")
            return compiled_graph
        except Exception as e:
            logger.error(f"Error building graph for BigToolAgent '{self.name}': {e}")
            raise

    @property
    def graph(self):
        return self._graph


__all__ = ["BigToolAgent"]



from universal_mcp.agentr.registry import AgentrRegistry
from universal_mcp.agents.builder import BuilderAgent
import json


def load_tasks():
    with open("src/evals/datasets/tasks.jsonl", "r") as f:
        for line in f:
            yield json.loads(line)


async def main():
    registry = AgentrRegistry()
    builder = BuilderAgent(
        name="Builder Agent",
        instructions="You are a builder agent that creates other agents.",
        model="gemini/gemini-1.5-pro",
        registry=registry,
    )
    updated_tasks = []
    tasks = load_tasks()
    for task in tasks:
        print(task["user_input"])
        result = await builder.invoke(task["user_input"])
        tools = result["tool_config"] or {}
        updated_tasks.append({**task, "required_tools": tools})
    with open("src/evals/datasets/tasks_with_tools.jsonl", "w") as f:
        for task in updated_tasks:
            f.write(json.dumps(task) + "\n")


if __name__ == "__main__":
    import asyncio
    asyncio.run(main())


from universal_mcp.agents.autoagent import AutoAgent
from universal_mcp.agents.base import BaseAgent
from universal_mcp.agents.bigtool import BigToolAgent
from universal_mcp.agents.bigtool2 import BigToolAgent2
from universal_mcp.agents.builder import BuilderAgent
from universal_mcp.agents.planner import PlannerAgent
from universal_mcp.agents.react import ReactAgent
from universal_mcp.agents.simple import SimpleAgent


def get_agent(agent_name: str):
    if agent_name == "auto":
        return AutoAgent
    elif agent_name == "react":
        return ReactAgent
    elif agent_name == "simple":
        return SimpleAgent
    elif agent_name == "builder":
        return BuilderAgent
    elif agent_name == "planner":
        return PlannerAgent
    elif agent_name == "bigtool":
        return BigToolAgent
    elif agent_name == "bigtool2":
        return BigToolAgent2
    else:
        raise ValueError(f"Unknown agent: {agent_name}. Possible values: auto, react, simple, builder, planner, bigtool, bigtool2")

__all__ = [
    "BaseAgent",
    "ReactAgent",
    "SimpleAgent",
    "AutoAgent",
    "BigToolAgent",
    "PlannerAgent",
    "BuilderAgent",
    "BigToolAgent2",
]
